"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generate = void 0;
// import { compile as compileTypescript } from 'json-schema-to-typescript';
const promises_1 = require("fs/promises");
const path_1 = require("path");
const prettier_1 = require("prettier");
const utils_1 = require("../utils");
const interfaces_1 = require("./interfaces");
const route_1 = require("./route");
const schemas_1 = require("./schemas");
async function generate(options, schema) {
    const interfaces = await (0, interfaces_1.generateInterface)(options, schema.routeGroups);
    const routes = await (0, route_1.generateRoutes)(options, schema.routeGroups);
    const schemas = await (0, schemas_1.generateSchemas)(options, schema.routeGroups);
    const groupNames = [...schema.routeGroups.keys()];
    await Promise.all(groupNames.map(async (groupName) => {
        const path = (0, path_1.resolve)(schema.version, options.outDirectory, groupName);
        console.log('Creating dirs...', path);
        await (0, promises_1.mkdir)(path, { recursive: true });
        const typesPath = (0, path_1.resolve)(path, 'types.ts');
        const groupInterfaces = interfaces.get(groupName);
        let typesContent = `
                /* eslint-disable @typescript-eslint/no-empty-interface */

                /**
                 * This file is auto-generated by . Please do not modify it manually.
                  */
            `;
        const interfaceNames = [];
        console.log('groupInterfaces', groupInterfaces);
        groupInterfaces.forEach(groupInterface => {
            typesContent += `
                    ${groupInterface.params || ''}

                    ${groupInterface.query || ''}

                    ${groupInterface.body || ''}

                    ${groupInterface.response || ''}
                `;
            interfaceNames.push(...[
                groupInterface.names.params,
                groupInterface.names.body,
                groupInterface.names.query,
                groupInterface.names.response,
            ].filter(Boolean));
        });
        console.log('typesContent=', typesContent);
        typesContent = (0, prettier_1.format)(typesContent, options.prettierOptions);
        console.log('Writing types.ts...', typesPath, typesContent.length, 'bytes');
        await (0, promises_1.writeFile)(typesPath, typesContent, { flag: 'w' });
        const routesPath = (0, path_1.resolve)(path, 'routes.ts');
        const routeCode = routes.get(groupName);
        const routesContent = (0, prettier_1.format)(`
                /**
                 * This file was initially auto-generated. Its content may be changed on the next generation.
                 */
                import { FastifyInstance, FastifyPluginAsync } from 'fastify';
                ${routeCode.imports}

                export const ${(0, utils_1.ucFirst)(groupName)}Routes: FastifyPluginAsync = async (fastify: FastifyInstance) => {
                    ${routeCode.inFunctionBody}
                };
            `, options.prettierOptions);
        console.log('Writing routes.ts...', routesPath, routesContent.length, 'bytes');
        await (0, promises_1.writeFile)(routesPath, routesContent, { flag: 'w' });
        const schemasPath = (0, path_1.resolve)(path, 'schemas.ts');
        const groupSchemas = schemas.get(groupName);
        const schemasContent = (0, prettier_1.format)(`
                /**
                 * This file is auto-generated by . Please do not modify it manually.
                 */
                ${groupSchemas.imports}

                ${groupSchemas.routes
            .map(e => `
                            export interface ${e.schemaTypeName} ${e.schemaTypeContent};
                            export const ${e.schemaObjectName} = ${e.schemaObjectContent};
                        `)
            .join('\n')}
            `, options.prettierOptions);
        console.log('Writing schemas.ts...', schemasPath, schemasContent.length, 'bytes');
        await (0, promises_1.writeFile)(schemasPath, schemasContent, { flag: 'w' });
    }));
}
exports.generate = generate;
