"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateRoutes = void 0;
const imports_1 = require("./imports");
const schemas_1 = require("./schemas");
async function generateRoutes(options, schema) {
    const groupCodes = new Map();
    const groupNames = [...schema.keys()];
    await Promise.all(groupNames.map(async (groupName) => {
        const groupRoutes = schema.get(groupName);
        let inFunctionBody = '';
        const imports = new imports_1.ImportsGenerator();
        groupRoutes.map(async (route) => {
            const prefix = options.schemeInterfaces.groupPrefix ? route.group : '';
            const schemaObjectName = (0, schemas_1.getRouteSchemaObjectName)(prefix, route);
            const schemaTypeName = (0, schemas_1.getRouteSchemaTypeName)(prefix, route);
            imports.addImport('./schemas', schemaObjectName);
            imports.addImport('./schemas', schemaTypeName);
            inFunctionBody += `
                    fastify.${route.method.toLowerCase()}<${schemaTypeName}>('${route.url}', {
                        handler: async (req, res) => {
                            // TODO: user code goes here
                            throw new Error('Not implemented');
                        },
                        schema: ${schemaObjectName},
                    })
                `;
        });
        groupCodes.set(groupName, {
            imports: imports.generate(),
            inFunctionBody
        });
    }));
    return groupCodes;
}
exports.generateRoutes = generateRoutes;
